/*
    Copyright (c) 2002 Jens Tinz
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions
    are met:
    1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
    3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
    IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
    OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
    IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
    NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
    THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/* Code used by the options dialog (options.xul). */

const XULNS = 'http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul';

var menuPreview = null;
var prefbranch = null;
var observerAdded = false;
// This should get reassigned before it is used, but initialize it to
// something valid (empty menu) just in case:
var _dummy = new MZPieMenu.Dummy();
var dummyLabel = '';
var currentLayout = [
    _dummy, _dummy, _dummy, _dummy,
    _dummy, _dummy, _dummy, _dummy,
];
delete _dummy;


// Helper functions:

function _addTreeCell(row, value) {
    var cell = row.appendChild(document.createElementNS(XULNS, 'treecell'));
    if (value !== null)
        cell.setAttribute('label', value);
}

/* Convert an item id like 'menu_tab' to a fancy string like 'menu -> Tab' */
function itemNameToDescription(name) {
    // HACK: specialcase this (because we do not want this label to
    // show up in the menu preview):
    if (name == 'dummy')
        return dummyLabel;
    var label = MZPieActions.items[name].label || name;
    var bits = name.split('_', 2);
    if (bits.length === 1)
        return label;

    // XXX getting the bundle this way *sucks*, refactor this once we
    // are a proper js module.
    var categoriesBundle = document.getElementById(
        'radialcontext_mz_categories');
    // u2192 is "rightwards arrow"
    // u21d2 is "rightwards double arrow"
    return categoriesBundle.getString(bits[0]) + ' \u21d2 ' + label;
}


// Code for the "general" pane:

function initBehavior(pane) {
    dragonlyChanged(document.getElementById('rc-dragonly-pref'));
}

function dragonlyChanged(pref) {
    document.getElementById('rc-click-for-normal').disabled = !pref.value;
}


// Code for the "customize" pane:

function initCustomize(pane) {
    // XXX see the comment at the top of options.xul
    var iframe = document.createElementNS(XULNS, 'iframe');
    iframe.setAttribute('id', 'demo');
    iframe.setAttribute('class', 'inset');
    iframe.setAttribute('flex', '1');
    iframe.setAttribute('height', '150');
    iframe.setAttribute('width', '340');
    document.getElementById('demo-container').appendChild(iframe);
    /* The plan: we want to support non-instantApply, so we need the
     * actions code to ignore the pref settings and apply our own current
     * customizations by hand. We store those as dom nodes in the xul
     * tree element we use. When the user hits apply/close we flush
     * stuff to prefs.
     *
     * To support instantApply we can do at least two things:
     * - implement nsITreeView to tie the tree to the pref system
     * - use a normal xul tree manually synced through a pref observer
     *
     * Because nsITreeView is very much not fun to implement (it is not
     * purely a data model thing like the gtk treeview has, there is
     * a bunch of bookkeeping to implement) we go for route two.
     * When the user edits something we instantapply it to the pref
     * system, which triggers our pref observer which updates the tree.
     *
     * We also need to store the uncustomized prefs somewhere. Currently we
     * just keep MZPieActions.layouts vanilla and apply customizations
     * before passing the layout to PieMenu.
     *
     * I am not 100% sure the ui as currently implemented will make sense
     * to new users. The idea is the selected item in the treeview and
     * the values in the menulists are synced. If the selection changes
     * the menus are updated. If the menu values change and there is a
     * selection the selected item is either updated or unselected.
     * If the menu values change and there is no selection that remains
     * the case (so randomly poking around in the menus will not select
     * and then update something in the tree).
     *
     * the menuPreview shows the menu from the "menu" menulist with
     * the customizations from the tree.
     *
     * The problem is deciding when to modify the selected item and when
     * to unselect it. Currently only changing the customized item updates.
     * Making the rest also update makes it easy to accidentally overwrite
     * a different customization. Downside is moving an item requires an
     * add/delete cycle (but it should be fairly easy to figure out:
     * - select current item
     * - modify "entry"
     * - hey, nothing happened (except for selection vanishing), lets hit add
     * - hey, now I have two, let's select the other and hit remove
     * )
     */
    var labelsBundle = document.getElementById('radialcontext_mz_labels');
    var menusBundle = document.getElementById('radialcontext_mz_menus');
    var categoriesBundle = document.getElementById(
        'radialcontext_mz_categories');
    MZPieActions.init(labelsBundle, null);

    // HACK: hack up the ContextMenu so we can show a special label for it.
    // Inserting this here (and not in MZPieActions.layouts) also makes it
    // not show up in our list of customizable menu.
    // TODO: bribe someone to create a custom icon for this one.
    var contextMenu = MZPieActions.items['context_2'];
    contextMenu.map['context'] = new MZPieMenu.Submenu(
        labelsBundle.getString('menu_context'), 'scroll/menu.png', []);
    // Call tweak now (with a mock context) to make sure the contextmenu
    // has a label set when we populate the tree.
    contextMenu.tweak({'next': {'layout': 'context'}});

    var menu = document.getElementById('customize-menu-menu');
    var items = [name for (name in MZPieActions.layouts)];
    items.sort();
    for each (var name in items) {
        menu.appendItem(menusBundle.getString(name), name);
    }
    // XXX Not 100% sure this is a "public" event
    // (using 'command' is inconvenient because it does not fire when we
    // change the selection programmatically here and when the tree
    // selection changes)
    menu.addEventListener('ValueChange', customizeOnLayoutSelect, true);
    menu.addEventListener('command', customizeOnCommandMenu, false);
    menu.selectedIndex = 0;

    menu = document.getElementById('customize-menu-entry');
    menu.addEventListener('command', customizeOnCommandEntry, false);
    menu = document.getElementById('customize-menu-item');
    // Specialcase the "dummy" item because we need to fix up its
    // label and because it is the only one not in any category.
    // HACK: itemNameToDescription needs this.
    dummyLabel = labelsBundle.getString('dummy');
    var dummyItem = menu.appendItem(dummyLabel, 'dummy');
    menu.selectedItem = dummyItem;
    var otheritems = [
        name for (name in MZPieActions.items) if (name !== 'dummy')];
    // This sorts by category as long as everything actually has a category.
    otheritems.sort();
    var lastCategory = null;
    for each (var itemname in otheritems) {
        var [category, rest] = itemname.split('_', 2);
        if (category !== lastCategory) {
            menu.menupopup.appendChild(
                document.createElementNS(XULNS, 'menuseparator'));
            var header = menu.appendItem(
                categoriesBundle.getString(category), category);
            header.disabled = true;
            header.className = 'header';
            // HACK: force the color to be normal non-disabled
            // (normally black) instead of disabled (gray). I still
            // want to keep the "disabled" attribute because selecting
            // a header makes no sense, but people are somewhat
            // conditioned to ignore disabled-gray entries completely
            // even when they are header-bold.
            header.style.color = 'menutext';
            lastCategory = category;
        }
        var item = MZPieActions.items[itemname];
        var menuEntry = menu.appendItem(item.label || itemname, itemname);
        // XXX this makes the icon show up on the menulist "button"
        // but not in the actual list. This seems to be intentional
        // (gnomestripe menu.css has a rule setting display to none
        // for "menulist > menupopup > menu > .menu-iconic-left",
        // which is the hbox the icon is in). I can work around that
        // by using a button type="menu" but that is a hassle (most of
        // the menupopup manipulation code we call is defined on the
        // menulist, not the menupopup). If we go that route we should
        // probably just use a full custom popup instead of a
        // menupopup so we can do pretty categories and whatnot too,
        // but that is a Firefox-3 feature, so will not happen yet.
        menuEntry.className = 'menuitem-iconic';
        // XXX HACK: not sure MZPieMenu.prototype.path should be
        // public at all.
        menuEntry.image = MZPieMenu.prototype.path + item.iconpath;
    }
    menu.addEventListener('command', customizeOnCommandItem, false);

    prefbranch = Components.classes[
        "@mozilla.org/preferences-service;1"].getService(
            Components.interfaces.nsIPrefService).getBranch(
                'extensions.radialcontext-mz.');
    prefbranch.QueryInterface(Components.interfaces.nsIPrefBranch2);
    var lastLayout = null;
    var tree = document.getElementById('customize-tree');
    var entries = document.getElementById(
        'customize-menu-entry').menupopup.childNodes;
    var children;

    for each (var key in prefbranch.getChildList('customizations.', {})) {
        var entry = MZPieActions.parseKey(prefbranch, key, false);
        if (entry !== null) {
            var [layoutName, index, itemName] = entry;
            // This relies on getChildList returning stuff sorted.
            if (lastLayout !== layoutName) {
                children = _customizeAddLayout(tree, layoutName);
                lastLayout = layoutName;
            }
            _customizeAddEntry(
                children, layoutName, index, itemName, entries);
        }
    }

    if (document.documentElement.instantApply) {
        prefbranch.addObserver('customizations.', customizeObserver, false);
        observerAdded = true;
    }
    tree.addEventListener('select', customizeOnTreeSelect, true);

    // XXX this code is glitchy/racy. Apparently (?) the iframe may or
    // may not be fully loaded by the time we get here. If I just do
    // the customizeOnDemoLoad now the iframe tends to be blank the
    // second time the pref pane is displayed. If I just add the
    // onload handler it is usually empty the first time. But
    // demoFrame.contentDocument is *not* null then, as far as I can
    // tell. So just do both for now, but I should probably
    // investigate what actually goes on here load order wise.
    var demoFrame = document.getElementById('demo');
    // Paranoia. No idea if this will ever not be true.
    if (demoFrame.contentDocument != null)
        customizeOnDemoLoad(demoFrame.contentDocument);
    demoFrame.addEventListener(
        'load', function(evt) {
            customizeOnDemoLoad(evt.currentTarget.contentDocument);
        }, true);
}

var customizeObserver = {
    observe: function(prefbranch_ignored, topic, key) {
        if (topic !== 'nsPref:changed')
            throw new Error('huh, I did not sign up for ' + topic);

        var entry = MZPieActions.parseKey(prefbranch, key, true);
        if (entry === null)
            return;

        var tree = document.getElementById('customize-tree');

        var [layoutName, index, itemName] = entry;
        if (itemName === null) {
            var columns = tree.columns;
            var view = tree.view;
            var indexCol = columns.getNamedColumn('customize-tree-index');
            var nameCol = columns.getNamedColumn('customize-tree-name');
            var layoutIndex = _customizeFindLayout(view, nameCol, layoutName);
            // If layoutIndex is -1 (should not happen) this runs 0 times.
            for (var i in _customizeFindLayoutChildren(view, layoutIndex)) {
                if (view.getCellText(i, indexCol) === String(index)) {
                    // Found it in the tree.
                    if (document.getElementById(
                        'customize-menu-menu').value === layoutName)
                        // We are currently previewing this menu.
                        _customizeRevertLayout(layoutName, index);

                    _customizeRemoveIndex(view, i, layoutIndex);
                    break;
                }
            }
        } else
            _customizeAdd(tree, layoutName, index, itemName);
    }
};

function _customizeApplyLayout(itemName, index) {
    currentLayout[index] = MZPieActions.items[itemName];
    if (menuPreview !== null)
        menuPreview.applyLayout(currentLayout);
}

function _customizeRevertLayout(layoutName, index) {
    currentLayout[index] = MZPieActions.layouts[layoutName][index];
    if (menuPreview !== null)
        menuPreview.applyLayout(currentLayout);
}

function _customizeRemoveIndex(view, index, parentIndex) {
    var item = view.getItemAtIndex(index);
    item.parentNode.removeChild(item);
    // And remove the parent if it has no items left:
    // XXX I want to use isContainer or isContainerEmpty here but
    // according to those it stays a nonempty container. Moz bug?
    item = view.getItemAtIndex(parentIndex);
    for each (var child in item.childNodes) {
        if (child.localName === 'treechildren' && child.hasChildNodes())
            return;
    }
    item.parentNode.removeChild(item);
}

function _customizeAdd(tree, layoutName, entry, itemName) {
    _customizeApplyLayout(itemName, entry);

    var view = tree.view;
    var columns = tree.columns;
    var indexCol = columns.getNamedColumn('customize-tree-index');
    var nameCol = columns.getNamedColumn('customize-tree-name');
    var layoutIndex = _customizeFindLayout(view, nameCol, layoutName);
    var entryIndex = -1;
    for (var i in _customizeFindLayoutChildren(view, layoutIndex)) {
        if (parseInt(view.getCellText(i, indexCol)) === entry) {
            // There is an existing entry, update it:
            view.setCellText(i, nameCol, itemName);
            view.setCellText(
                i, columns.getNamedColumn('customize-tree-customized'),
                itemNameToDescription(itemName));
            entryIndex = i;
            break;
        }
    }
    if (entryIndex === -1) {
        // There was no existing entry, create one.
        // First create the parent node if it does not exist yet:
        var children;
        if (layoutIndex === -1)
            children = _customizeAddLayout(tree, layoutName);
        else
            // It exists, so it has at least one child, which we can abuse
            // to get at its "children" node:
            children = view.getItemAtIndex(layoutIndex + 1).parentNode;
        var item = _customizeAddEntry(
            children, layoutName, entry, itemName, null);
        entryIndex = view.getIndexOfItem(item);
    }
    // Update tree selection if there is none.
    // Notice this fires a select event, so we should do it after
    // updating the values in the tree or that event handler will
    // modify the menu values.
    if (view.selection.currentIndex === -1 ||
        view.getParentIndex(view.selection.currentIndex) === -1)
        view.selection.select(entryIndex);
}

function _customizeAddLayout(tree, layoutName) {
    var item = tree.body.appendChild(
        document.createElementNS(XULNS, 'treeitem'));
    item.setAttribute('container', 'true');
    item.setAttribute('open', 'true');
    var row = document.createElementNS(XULNS, 'treerow');
    var bundle = document.getElementById('radialcontext_mz_menus');
    // Name as displayed in the first column:
    _addTreeCell(row, bundle.getString(layoutName));
    // "original", "customized" and "entry" are empty:
    for (var i = 0; i < 3; ++i)
        _addTreeCell(row, null);
    // "name" is the layout name (currently identical to the first item,
    // but that may become a fancier string in the future)
    _addTreeCell(row, layoutName);
    item.appendChild(row);
    return item.appendChild(document.createElementNS(XULNS, 'treechildren'));
}

function _customizeAddEntry(children, layoutName, index, itemName, entries) {
    if (entries === null)
        entries = document.getElementById(
            'customize-menu-entry').menupopup.childNodes;
    var item = children.appendChild(
        document.createElementNS(XULNS, 'treeitem'));
    var row = item.appendChild(document.createElementNS(XULNS, 'treerow'));

    _addTreeCell(row, entries[index].getAttribute('label'));

    // XXX this is sucky, do we want to make this easier? id an
    // attr of the action? I'd say "map from action back to id"
    // but object/array only take string/int keys, so...
    var origAction = MZPieActions.layouts[layoutName][index];
    var done = false;
    for (var [name, action] in Iterator(MZPieActions.items)) {
        if (action === origAction) {
            _addTreeCell(row, itemNameToDescription(name));
            done = true;
            break;
        }
    }
    if (!done)
        throw new Error('reverse map from action to id failed?');

    _addTreeCell(row, itemNameToDescription(itemName));

    // XXX cells have a convenient "value" attribute but I do not
    // see a sane way to get at it from the nsITreeView, so I use
    // hidden columns instead.
    _addTreeCell(row, String(index));
    _addTreeCell(row, itemName);
    return item;
}

/* Update the menus (which updates the preview if necessary). */
function customizeOnTreeSelect(evt) {
    var tree = evt.target;
    var view = tree.view;
    var columns = tree.columns;
    var addButton = document.getElementById('customize-add');
    var removeButton = document.getElementById('customize-remove');
    var index = view.selection.currentIndex;
    if (index === -1) {
        addButton.disabled = false;
        removeButton.disabled = true;
        return;
    }

    var menu = document.getElementById('customize-menu-menu');
    var indexCol = columns.getNamedColumn('customize-tree-index');
    var nameCol = columns.getNamedColumn('customize-tree-name');
    if (view.isContainer(index)) {
        // menu name selected
        menu.value = view.getCellText(index, nameCol);
        addButton.disabled = false;
        removeButton.disabled = true;
    } else {
        addButton.disabled = true;
        removeButton.disabled = false;
        // menu entry selected
        menu.value = view.getCellText(view.getParentIndex(index), nameCol);

        menu = document.getElementById('customize-menu-entry');
        menu.selectedIndex = parseInt(view.getCellText(index, indexCol));

        menu = document.getElementById('customize-menu-item');
        menu.value = view.getCellText(index, nameCol);
    }
}

/* Add a PieMenu to the iframe once it has loaded. */
function customizeOnDemoLoad(idoc) {
    // idoc is an XPCNativeWrapper! That is probably ok though.
    menuPreview = new MZPieMenu(
        idoc.body, currentLayout, 170, 70,
        document.getElementById('customize-altbox').checked, true, 1, {
            frameDoc: idoc,
            image: null,
            selection: null,
            link: null,
            // HACK: special child context to make the context_2 item show up
            // (this is not complete enough to work as an actual context,
            // but only ContextMenu looks at this)
            next: {
                layout: 'context'
            }
        });
    menuPreview.showLabels();
}

/* Helper returning the index of the node for a layout name or -1. */
function _customizeFindLayout(view, nameCol, layoutName) {
    if (!view.rowCount)
        return -1;
    var index = 0;
    while (true) {
        if (view.getCellText(index, nameCol) === layoutName)
            return index;
        // I do not see a "next sibling" method in nsITreeView so
        // take a detour through the DOM:
        var next = view.getItemAtIndex(index).nextSibling;
        if (next === null)
            return -1;
        index = view.getIndexOfItem(next);
    }
}

/* Generator yielding tree indices for a layout.
 *
 * index is the result from _customizeFindLayout.
 */
function _customizeFindLayoutChildren(view, index) {
    if (index === -1)
        return;
    for (++index; index < view.rowCount && view.getLevel(index); ++index)
        yield index;
}

/* Update the menu preview to match the selected menu. */
function customizeOnLayoutSelect(evt) {
    // Make a copy of the layout since we will apply customizations to it:
    var value = evt.target.value;
    var layout = MZPieActions.layouts[value].slice(0);

    // apply customizations
    var tree = document.getElementById('customize-tree');
    var view = tree.view;
    var columns = tree.columns;
    var indexCol = columns.getNamedColumn('customize-tree-index');
    var nameCol = columns.getNamedColumn('customize-tree-name');
    var index = _customizeFindLayout(view, nameCol, value);
    for (var i in _customizeFindLayoutChildren(view, index)) {
        layout[view.getCellText(i, indexCol)] =
            MZPieActions.items[view.getCellText(i, nameCol)];
    }

    currentLayout = layout;
    if (menuPreview !== null)
        menuPreview.applyLayout(layout);
}

function _customizeSetPref(layoutName, index, itemName) {
    prefbranch.setCharPref(
        'customizations.' + layoutName + '.' + String(index), itemName);
}

function customizeOnCommandMenu(evt) {
    // Lose the selection in the tree if there is one.
    // (could change the menu the current item is in but that is probably
    // unwanted, especially since it could overwrite something else)

    // Beware: the documentation
    // http://developer.mozilla.org/en/docs/XUL_Tutorial:Tree_Selection
    // is slightly misleading here:
    // although it mentions you should not assign to currentIndex to change
    // the selection, it does not say why or how to properly unselect stuff.
    // The reason is hinted at in nsITreeSelection.idl: currentIndex is not
    // the *selected* row, it is the *current* or *focused* row (the one with
    // a focus rect around it). *Normally* that row is also selected (and
    // vice versa), but this is not guaranteed. Specifically clearSelection
    // does not touch currentIndex! Result:
    // - Tree has a row selected
    // - User changes menuentry (this function runs)
    // - This calls clearSelection
    // - clearSelection visually clears the selection but leaves currentIndex
    // - clearSelection fires a select event (customizeOnTreeSelect runs)
    // - that uses currentIndex to set this menulist back to what it thinks
    //   is the currently selected layout
    // so it looks like the menulist breaks.
    // Conclusion: in single-select mode *only* use the select method
    // to manipulate the selection, since that one updates both currentIndex
    // and the selection.
    document.getElementById('customize-tree').view.selection.select(-1);
}

function customizeOnCommandEntry(evt) {
    document.getElementById('customize-tree').view.selection.select(-1);
}

function customizeOnCommandItem(evt) {
    // Update the selected item if there is one.
    var tree = document.getElementById('customize-tree');
    var view = tree.view;
    var columns = tree.columns;
    var treeIndex = view.selection.currentIndex;
    if (treeIndex === -1)
        return;

    var parent = view.getParentIndex(treeIndex);
    if (parent === -1)
        return;

    var itemName = evt.target.value;
    var nameCol = columns.getNamedColumn('customize-tree-name');
    var entry = view.getCellText(
        treeIndex, columns.getNamedColumn('customize-tree-index'));

    if (document.documentElement.instantApply) {
        _customizeSetPref(view.getCellText(parent, nameCol), entry, itemName);
    } else {
        // Update the tree and preview:
        view.setCellText(
            treeIndex, columns.getNamedColumn('customize-tree-customized'),
            itemNameToDescription(itemName));
        view.setCellText(treeIndex, nameCol, itemName);
        _customizeApplyLayout(itemName, parseInt(entry));
    }
}

function customizeOnAdd() {
    // This should be effectively a noop if there is a selection.
    var layoutName = document.getElementById('customize-menu-menu').value;
    var index = document.getElementById('customize-menu-entry').selectedIndex;
    var itemName = document.getElementById('customize-menu-item').value;
    if (document.documentElement.instantApply)
        _customizeSetPref(layoutName, index, itemName);
    else
        _customizeAdd(document.getElementById('customize-tree'),
                      layoutName, index, itemName);
}

function customizeOnRemove() {
    var tree = document.getElementById('customize-tree');
    var view = tree.view;
    var columns = tree.columns;
    var index = view.selection.currentIndex;

    // Ignore attempts to remove without a customization item selected.
    if (index === -1)
        return;

    // TODO: maybe support resetting an entire menu to default this way?
    var parent = view.getParentIndex(index);
    if (parent === -1)
        return;

    var layoutName = view.getCellText(
        parent, columns.getNamedColumn('customize-tree-name'));
    var layoutIndex = view.getCellText(
        index, columns.getNamedColumn('customize-tree-index'));

    if (document.documentElement.instantApply)
        prefbranch.clearUserPref(
            'customizations.' + layoutName + '.' + String(layoutIndex));
    else {
        _customizeRevertLayout(layoutName, layoutIndex);
        _customizeRemoveIndex(view, index, parent);
    }
}

function customizeAccept() {
    // Careful: this is hooked up to a dialog event handler, not a pane
    // event handler, so should not rely on initCustomize having run!
    if (!document.documentElement.instantApply && prefbranch !== null) {
        var tree = document.getElementById('customize-tree');
        var view = tree.view;
        var columns = tree.columns;
        var nameCol = columns.getNamedColumn('customize-tree-name');
        var indexCol = columns.getNamedColumn('customize-tree-index');

        // This should be resetBranch but that returns "not implemented".
        // Then I tried deleteBranch but apparently that does not fire
        // pref observers. So do it by hand:
        for each (var key in prefbranch.getChildList('customizations.', {})) {
            prefbranch.clearUserPref(key);
        }
        prefbranch.deleteBranch('customizations.');
        var layoutName = null;
        for (var i = 0; i < view.rowCount; ++i) {
            if (!view.getLevel(i))
                layoutName = view.getCellText(i, nameCol);
            else
                _customizeSetPref(layoutName, view.getCellText(i, indexCol),
                                  view.getCellText(i, nameCol));
        }
    }
    customizeCleanup();
    return true;
}

function customizeCleanup() {
    if (prefbranch !== null) {
        // We use an explicit flag to track whether we added an
        // observer instead of checking the instantApply pref here to
        // prevent a hole when the instantApply pref is toggled after
        // adding the observer.
        if (observerAdded) {
            prefbranch.removeObserver('customizations.', customizeObserver);
            observerAdded = false;
        }
        prefbranch = null;
    }
}

function customizeToggleAlt(status) {
    if (menuPreview !== null)
        menuPreview.updateEntries(status);
}

// Code for the "about" pane:

function initAbout(prefpane) {
    // XXX See the comment at the top of options.xul.
    var iframe = document.createElementNS(XULNS, 'iframe');
    iframe.setAttribute('src',
                        'chrome://radialcontext-mz/content/about.html');
    iframe.setAttribute('class', 'box-inset');
    iframe.setAttribute('minheight', '400');
    iframe.setAttribute('flex', '1');
    prefpane.appendChild(iframe);
}
