/*
    Copyright (c) 2002 Jens Tinz
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions
    are met:
    1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
    3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
    IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
    OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
    IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
    NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
    THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/* Code used by the tip of the day dialog (tipdialog.xul) */

var TipDialog = {
    prefbranch: null,
    /* Could just use the pref value, but keeping a local copy makes
     * multiple tips dialogs simultaneously behave a bit saner. */
    tipIndex: null,
    bundle: null,
    tipcount: 0,

    /* Initialize members and hook up event handlers. */
    init: function() {
        if (this.prefbranch !== null)
            throw new Error('huh, TipDialog inited twice?');
        this.bundle = document.getElementById('rcbundle');
        this.tipcount = parseInt(this.bundle.getString('tipcount'));
        this.prefbranch = Components.classes[
            '@mozilla.org/preferences-service;1']
            .getService(Components.interfaces.nsIPrefService)
            .getBranch('extensions.radialcontext-mz.')
            .QueryInterface(Components.interfaces.nsIPrefBranch2);
        this.prefbranch.addObserver('tips_at_startup', this, false);
        this.tipIndex = this.prefbranch.getIntPref('tip_nr') % this.tipcount;
        /* Older versions set this to -1 for "no tips at startup"
         * (and -1 % n === -1, apparently). */
        if (this.tipIndex < 0)
            this.tipIndex = 0;
        /* Fake a "change" to initialize the checkbox. */
        this.observe(this.prefbranch, 'nsPref:changed', 'tips_at_startup');
        this.displayTip();
    },

    /* Unregister event handlers registered in init(). */
    shutdown: function() {
        this.prefbranch.removeObserver('tip_nr', this);
    },

    /* Handle a preference change. */
    observe: function(subject, topic, data) {
        if (topic !== 'nsPref:changed')
            return;
        // data should always be the same, but let's be future-proof/paranoid.
        switch (data) {
        case 'tips_at_startup':
            document.getElementById('tip-show').checked = (
                this.prefbranch.getBoolPref('tips_at_startup'));
            break;
        }
    },

    /* Show the next tip. */
    displayTip: function() {
        var str = this.bundle.getFormattedString(
            'tip_nr', [this.tipIndex + 1, this.tipcount]);
        document.getElementById('tip-nr-display').setAttribute('value', str);
        str = this.bundle.getString('tip.' + (this.tipIndex + 1));
        document.getElementById('tip-content').textContent = str;
        this.tipIndex = ++this.tipIndex % this.tipcount;
        this.prefbranch.setIntPref('tip_nr', this.tipIndex);
    },

    /* Event handler for the "show tips at startup" checkbox. */
    setTipActivation: function(activate) {
        this.prefbranch.setBoolPref('tips_at_startup', activate);
    }
};


window.addEventListener('load', function() { TipDialog.init(); }, false);
window.addEventListener(
    'unload', function() { TipDialog.shutdown(); }, false);
